/*
 * Copyright (c) 1994-1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL or COMMERCIAL purposes and
 * without fee is hereby granted.
 * Please refer to the file http://java.sun.com/copy_trademarks.html
 * for further important copyright and trademark information and to
 * http://java.sun.com/licensing.html for further important licensing
 * information for the Java (tm) Technology.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

import java.awt.*;

public class ImageTape extends java.applet.Applet implements Runnable {
    Image imgs[];
    int nimgs;
    int imgwidth;
    int x, newx;
    int dist = 5;
    int timeout = 200;
    Thread scroller;
    String dir;

    public void init() {
	String at = getParameter("img");
	dir = (at != null) ? at : "doc:/demo/images/duke";
	at = getParameter("speed");
	timeout = 1000 / ((at == null) ? 4 : Integer.valueOf(at).intValue());
	at = getParameter("dir");
	dist = (at == null) ? 5 : Integer.valueOf(at).intValue();
	at = getParameter("nimgs");
	nimgs = (at == null) ? 35 : Integer.valueOf(at).intValue();
	newx = x = size().width;

	imgs = new Image[nimgs];
	for (int i = 0 ; i < nimgs ; i++) {
	    imgs[i] = getImage(getDocumentBase(), dir + "/T" + (i+1) + ".gif");
	}
    }

    public void start() {
	if (scroller == null) {
	    scroller = new Thread(this);
	    scroller.start();
	}
    }
    public void stop() {
	if (scroller != null) {
	    scroller.stop();
	    scroller = null;
	}
    }

    public void run() {
	while (true) {
	try {Thread.currentThread().sleep(timeout);} catch (InterruptedException e){}
	    scroll(dist);
	}
    }

    synchronized void scroll(int dist) {
	newx += dist;
	repaint();
    }

    public void update(Graphics g) {
	g.setColor(Color.lightGray);

	if (newx != x) {
	    //g.clipRect(1, 1, size().width-2, size().height-2);
	    int dist = newx - x;
	    if (dist > 0) {
		g.copyArea(1, 1, (size().width-2) - dist, size().height-2, dist, 0);
		for (x = newx ; x > size().width ; x -= Math.max(size().width - 2, imgwidth));
		paint(g, 1, dist + 1);
	    } else {
		g.copyArea(1 - dist, 1, (size().width-2) + dist, size().height-2, dist, 0);
		for (x = newx ; x < 0 ; x += Math.max(size().width - 2, imgwidth));
		paint(g, (size().width-1) + dist, size().width-1);
	    }
	} else {
	    paint(g);
	}
    }

    public boolean imageUpdate(Image img, int flags, int x, int y, int w, int h) {
	if ((flags & WIDTH) != 0) {
	    imgwidth += img.getWidth(this);
	}
	return super.imageUpdate(img, flags, x, y, w, h);
    }

    public void paint(Graphics g, int fromx, int tox) {
	int x = this.x;
	newx = x;

	g.setColor(Color.lightGray);
	g.fillRect(fromx, 1, tox - fromx, size().height-2);
	g.clipRect(fromx, 1, tox - fromx, size().height-2);
	g.setColor(Color.black);

	for (int i = 0 ; i < nimgs ; i++) {
	    if (imgs[i] == null) {
		continue;
	    }

	    int w = imgs[i].getWidth(this);
	    int h = imgs[i].getHeight(this);
	    if ((w > 0) && (h > 0)) {
		if ((x + w > fromx) && (x < tox)) {
		    g.drawImage(imgs[i], x, size().height - (h+1), this);
		}
		if ((x + w) > size().width) {
		    x -= Math.max(size().width - 2, imgwidth);
		    if ((x + w > fromx) && (x < tox)) {
			g.drawImage(imgs[i], x, size().height - (h + 1), this);
		    }
		}
		x += w;
	    }
	}
    }
    public void paint(Graphics g) {
	g.draw3DRect(0, 0, size().width, size().height, true);
	paint(g, 1, size().width-1);
    }
}
